@@ -2,6 +2,8 @@ use crate::prelude::*;
 
 use std::fmt::Display;
 
+use super::{article_view, comment_view, search_view, story_view};
+
 /// HelpView is a View displaying a help dialog with a list of key shortcuts and descriptions
 /// grouped by certain categories.
 pub struct HelpView {
@@ -21,10 +23,7 @@ impl HelpView {
     fn construct_key_view(key: String, desc: String, max_key_width: usize) -> impl View {
         let key_string = StyledString::styled(
             key,
-            ColorStyle::new(
-                PaletteColor::TitlePrimary,
-                get_config_theme().code_block_bg.color,
-            ),
+            config::get_config_theme().component_style.single_code_block,
         );
         let desc_string = StyledString::plain(desc);
         LinearLayout::horizontal()
@@ -33,7 +32,7 @@ impl HelpView {
     }
 
     fn construct_help_dialog_event_view(view: Dialog) -> OnEventView<Dialog> {
-        OnEventView::new(view).on_event(get_global_keymap().close_dialog.clone(), |s| {
+        OnEventView::new(view).on_event(config::get_global_keymap().close_dialog.clone(), |s| {
             s.pop_layer();
         })
     }
@@ -68,8 +67,8 @@ impl HelpView {
             .with(|s| {
                 self.key_groups.iter().for_each(|(group_desc, keys)| {
                     s.add_child(TextView::new(StyledString::styled(
-                        group_desc.to_string(),
-                        ColorStyle::from(PaletteColor::TitlePrimary),
+                        *group_desc,
+                        config::get_config_theme().component_style.bold,
                     )));
                     s.add_child(HelpView::construct_keys_view(keys));
                 });
@@ -119,9 +118,9 @@ macro_rules! other_key_shortcuts {
                 $(
                     ($k, $d),
                 )*
-                (get_global_keymap().open_help_dialog.to_string(), "Open the help dialog"),
-                (get_global_keymap().quit.to_string(), "Quit the application"),
-                (get_global_keymap().close_dialog.to_string(), "Close a dialog"),
+                (config::get_global_keymap().open_help_dialog.to_string(), "Open the help dialog"),
+                (config::get_global_keymap().quit.to_string(), "Quit the application"),
+                (config::get_global_keymap().close_dialog.to_string(), "Close a dialog"),
             ],
         )
     };
@@ -136,13 +135,13 @@ macro_rules! view_navigation_key_shortcuts {
                 $(
                     ($k, $d),
                 )*
-                    (get_global_keymap().goto_previous_view.to_string(), "Go to the previous view"),
-                    (get_global_keymap().goto_front_page_view.to_string(), "Go to front page view"),
-                    (get_global_keymap().goto_search_view.to_string(), "Go to search view"),
-                    (get_global_keymap().goto_all_stories_view.to_string(), "Go to all stories view"),
-                    (get_global_keymap().goto_ask_hn_view.to_string(), "Go to ask HN view"),
-                    (get_global_keymap().goto_show_hn_view.to_string(), "Go to show HN view"),
-                    (get_global_keymap().goto_jobs_view.to_string(), "Go to jobs view"),
+                    (config::get_global_keymap().goto_previous_view.to_string(), "Go to the previous view"),
+                    (config::get_global_keymap().goto_search_view.to_string(), "Go to search view"),
+                    (config::get_global_keymap().goto_front_page_view.to_string(), "Go to front page view"),
+                    (config::get_global_keymap().goto_all_stories_view.to_string(), "Go to all stories view"),
+                    (config::get_global_keymap().goto_ask_hn_view.to_string(), "Go to ask HN view"),
+                    (config::get_global_keymap().goto_show_hn_view.to_string(), "Go to show HN view"),
+                    (config::get_global_keymap().goto_jobs_view.to_string(), "Go to jobs view"),
             ],
 
         )
@@ -163,17 +162,18 @@ pub struct DefaultHelpView {}
 
 impl HasHelpView for DefaultHelpView {}
 
-impl HasHelpView for StoryView {
+impl HasHelpView for story_view::StoryView {
     fn construct_help_view() -> HelpView {
-        let story_view_keymap = get_story_view_keymap();
-        let custom_keymaps: Vec<(String, String)> = get_custom_keymap()
-            .custom_view_navigation
+        let story_view_keymap = config::get_story_view_keymap();
+        let custom_keymaps: Vec<(String, String)> = config::get_config()
+            .keymap
+            .custom_keymaps
             .iter()
             .map(|keymap| {
                 (
                     keymap.key.to_string(),
                     format!(
-                        "Go to {} view (sort_by: {}, {})",
+                        "Go to {} view (by_date: {}, {})",
                         match keymap.tag.as_str() {
                             "front_page" => "front page",
                             "story" => "all stories",
@@ -182,7 +182,7 @@ impl HasHelpView for StoryView {
                             "show_hn" => "show HN",
                             _ => panic!("unknown view: {}", keymap.tag),
                         },
-                        if keymap.by_date { "date" } else { "popularity" },
+                        keymap.by_date,
                         keymap.numeric_filters.desc()
                     ),
                 )
@@ -219,8 +219,8 @@ impl HasHelpView for StoryView {
                         "Go the previous page",
                     ),
                     (
-                        story_view_keymap.toggle_sort_by.to_string(),
-                        "Toggle sort by date/popularity (only for non `Front Page` views)",
+                        story_view_keymap.toggle_sort_by_date.to_string(),
+                        "Toggle sort by date (only for non `Front Page` views)",
                     ),
                 ],
             ),
@@ -246,19 +246,29 @@ impl HasHelpView for StoryView {
             help_view = help_view.key_groups(vec![("Custom keymaps", custom_keymaps)]);
         }
         help_view.key_groups(vec![
-            view_navigation_key_shortcuts!((
-                story_view_keymap.goto_story_comment_view.to_string(),
-                "Go to the comment view associated with the focused story"
-            )),
+            view_navigation_key_shortcuts!(
+                (
+                    story_view_keymap.goto_story_comment_view.to_string(),
+                    "Go to the comment view associated with the focused story"
+                ),
+                (
+                    story_view_keymap.next_story_tag.to_string(),
+                    "Go to the next story tag"
+                ),
+                (
+                    story_view_keymap.prev_story_tag.to_string(),
+                    "Go to the previous story tag"
+                )
+            ),
             other_key_shortcuts!(),
         ])
     }
 }
 
-impl HasHelpView for CommentView {
+impl HasHelpView for comment_view::CommentView {
     fn construct_help_view() -> HelpView {
-        let comment_view_keymap = get_comment_view_keymap();
-        let story_view_keymap = get_story_view_keymap();
+        let comment_view_keymap = config::get_comment_view_keymap();
+        let story_view_keymap = config::get_story_view_keymap();
 
         HelpView::new().key_groups(vec![
             (
@@ -342,24 +352,18 @@ impl HasHelpView for CommentView {
                 ],
             ),
             view_navigation_key_shortcuts!(),
-            other_key_shortcuts!(
-                (
-                    comment_view_keymap.reload_comment_view.to_string(),
-                    "Reload the comment view"
-                ),
-                (
-                    comment_view_keymap.toggle_collapse_comment.to_string(),
-                    "Toggle collapsing the focused comment"
-                )
-            ),
+            other_key_shortcuts!((
+                comment_view_keymap.toggle_collapse_comment.to_string(),
+                "Toggle collapsing the focused comment"
+            )),
         ])
     }
 }
 
-impl HasHelpView for SearchView {
+impl HasHelpView for search_view::SearchView {
     fn construct_help_view() -> HelpView {
-        let search_view_keymap = get_search_view_keymap();
-        let story_view_keymap = get_story_view_keymap();
+        let search_view_keymap = config::get_search_view_keymap();
+        let story_view_keymap = config::get_story_view_keymap();
 
         HelpView::new().key_groups(vec![
             (
@@ -404,8 +408,8 @@ impl HasHelpView for SearchView {
                         "Go the previous page",
                     ),
                     (
-                        story_view_keymap.toggle_sort_by.to_string(),
-                        "Toggle sort by date/popularity",
+                        story_view_keymap.toggle_sort_by_date.to_string(),
+                        "Toggle sort by date",
                     ),
                 ],
             ),
@@ -435,9 +439,9 @@ impl HasHelpView for SearchView {
     }
 }
 
-impl HasHelpView for ArticleView {
+impl HasHelpView for article_view::ArticleView {
     fn construct_help_view() -> HelpView {
-        let article_view_keymap = get_article_view_keymap().clone();
+        let article_view_keymap = config::get_article_view_keymap().clone();
         HelpView::new().key_groups(vec![
             (
                 "Navigation",
@@ -505,10 +509,6 @@ impl HasHelpView for ArticleView {
                 ],
             ),
             view_navigation_key_shortcuts!(),
-            other_key_shortcuts!((
-                article_view_keymap.toggle_raw_markdown_mode.to_string(),
-                "Toggle raw markdown mode"
-            )),
         ])
     }
 }
